extern "C" {
#include "./render.h"
#include "./reciter.h"
#include "./sam.h"
}

#include <Pokitto.h>
#include <LibAudio>

extern "C" void SamOutputByte(unsigned int pos, unsigned char data) {
    static unsigned char *buffer;
    static unsigned int bufferWriteOffset = 0;
    constexpr int factor = PROJ_AUD_FREQ / 8000 ?: 1;
    for(int i=0; i<factor; ++i){
        if(bufferWriteOffset >= 512){
            buffer = nullptr;
        }
    
        while(!buffer) {
            for(int i=0; i<Audio::bufferCount; ++i){
                if(Audio::audio_state[i] == 0){
                    Audio::audio_state[i] = 1;
                    buffer = Audio::audio_buffer + 512 * i;
                    bufferWriteOffset = 0;
                }
            }
        }
    
        buffer[bufferWriteOffset++] = data;
    }
}

void translate(const char *txt, reciter_memory& mem) {
    uint32_t len = strlen(txt), outlen;
    // Reciter truncates *output* at about 120 characters.
    // So to avoid that we must disallow any input that will exceed that.
    if (len > 80) {
        len = 80;
    }
    MemOps::set(mem.input, 0, 128);
    for (int i = 0; i < len; i++) {
        mem.input[i] = txt[i];
    }
    mem.input[len] = '[';

    if (!TextToPhonemes(&mem)) {
        LOG("could not parse input\n");
    }

    for (outlen = 0; outlen < 255; outlen++) {
        if (mem.input[outlen] == 155) {
            mem.input[outlen] = 0;
        }
    }
}

void articulate(const char *text, uint32_t pitch, uint32_t speed, uint32_t mouth, uint32_t throat, bool sing) {
    sam_memory sam;
    sam.common.singmode = sing;
    sam.common.pitch  = pitch;
    sam.common.speed  = speed;
    sam.common.mouth  = mouth; 
    sam.common.throat = throat;
    sam.prepare.input = text;
    sam.prepare.input_length = strlen(text);
    SAMMain(&sam);
}

namespace TTS {
    
void say(const char *text, 
    uint32_t pitch, 
    uint32_t speed, 
    uint32_t mouth, 
    uint32_t throat, 
    bool sing) {
        Pokitto::Core::updateHook(false);
        reciter_memory mem;
        translate(text, mem);
        articulate(mem.input, pitch, speed, mouth, throat, sing);
}

}